Class: Metanorma::FileLookup
- Inherits:
-
Object
- Object
- Metanorma::FileLookup
- Defined in:
- lib/metanorma/files_lookup.rb,
lib/metanorma/files_lookup_sectionsplit.rb
Overview
XML collection renderer
Instance Attribute Summary collapse
-
#files_to_delete ⇒ Object
Returns the value of attribute files_to_delete.
-
#parent ⇒ Object
Returns the value of attribute parent.
Instance Method Summary collapse
- #add_document_suffix(identifier, doc) ⇒ Object
- #add_section_split ⇒ Object
- #add_section_split_cover(manifest, ident) ⇒ Object
- #add_section_split_instance(file, manifest, key, idx) ⇒ Object
- #add_section_split_instance_prep(file, key) ⇒ Object
- #bibdata_process(entry, ident) ⇒ Object
- #bibitem_process(entry) ⇒ Object
- #cleanup_section_split_instance(key, manifest) ⇒ Object
- #each ⇒ Object
- #each_with_index ⇒ Object
-
#file_entry(ref, identifier) ⇒ Object
rel_path is the source file address, determined relative to the YAML.
- #file_entry_copy(ref, ret) ⇒ Object
- #get(ident, attr = nil) ⇒ Object
-
#initialize(path, parent) ⇒ FileLookup
constructor
hash for each document in collection of document identifier to: document reference (fileref or id), type of document reference, and bibdata entry for that file.
- #key(ident) ⇒ Object
- #keys ⇒ Object
- #ns(xpath) ⇒ Object
- #one_doc_collection? ⇒ Boolean
- #process_section_split_instance(key, manifest) ⇒ Object
-
#read_anchors(xml) ⇒ Object
map locality type and label (e.g. “clause” “1”) to id = anchor for a document Note: will only key clauses, which have unambiguous reference label in locality.
- #read_anchors1(key, val, ret) ⇒ Object
- #read_file(docref) ⇒ Object
- #read_files ⇒ Object
-
#read_ids(xml) ⇒ Object
Also parse all ids in doc (including ones which won’t be xref targets).
- #ref_file(ref, out, read, doc) ⇒ Object
- #sectionsplit(file, outfile, ident) ⇒ Object
- #set(ident, attr, value) ⇒ Object
-
#targetfile(data, options) ⇒ Array<String, nil>
return file contents + output filename for each file in the collection, given a docref entry so my URL should end with html or pdf or whatever formed relative to YAML file, not input path, relative to calling function.
- #targetfile_id(ident, options) ⇒ Object
-
#url(ident, options) ⇒ Object
return citation url for file so my URL should end with html or pdf or whatever.
-
#url?(ident) ⇒ Boolean
are references to the file to be linked to a file in the collection, or externally? Determines whether file suffix anchors are to be used.
-
#url_in_css_styles(doc, document_suffix) ⇒ Object
update relative URLs, url(#…), in CSS in @style attrs (including SVG).
- #xml_file(id, read) ⇒ Object
Constructor Details
#initialize(path, parent) ⇒ FileLookup
hash for each document in collection of document identifier to: document reference (fileref or id), type of document reference, and bibdata entry for that file
15 16 17 18 19 20 21 22 23 24 25 26 |
# File 'lib/metanorma/files_lookup.rb', line 15 def initialize(path, parent) @c = HTMLEntities.new @files = {} @parent = parent @xml = parent.xml @isodoc = parent.isodoc @path = path @compile = parent.compile @documents = parent.documents @files_to_delete = [] read_files end |
Instance Attribute Details
#files_to_delete ⇒ Object
Returns the value of attribute files_to_delete.
9 10 11 |
# File 'lib/metanorma/files_lookup.rb', line 9 def files_to_delete @files_to_delete end |
#parent ⇒ Object
Returns the value of attribute parent.
9 10 11 |
# File 'lib/metanorma/files_lookup.rb', line 9 def parent @parent end |
Instance Method Details
#add_document_suffix(identifier, doc) ⇒ Object
87 88 89 90 91 92 93 94 95 96 |
# File 'lib/metanorma/files_lookup.rb', line 87 def add_document_suffix(identifier, doc) document_suffix = Metanorma::Utils::to_ncname(identifier) Metanorma::Utils::anchor_attributes.each do |(tag_name, attribute_name)| Util::add_suffix_to_attributes(doc, document_suffix, tag_name, attribute_name, @isodoc) end url_in_css_styles(doc, document_suffix) doc.root["document_suffix"] ||= "" doc.root["document_suffix"] += document_suffix end |
#add_section_split ⇒ Object
4 5 6 7 8 9 10 11 12 13 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 4 def add_section_split ret = @files.keys.each_with_object({}) do |k, m| if @files[k][:sectionsplit] == "true" && !@files[k]["attachment"] process_section_split_instance(k, m) cleanup_section_split_instance(k, m) end m[k] = @files[k] end @files = ret end |
#add_section_split_cover(manifest, ident) ⇒ Object
31 32 33 34 35 36 37 38 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 31 def add_section_split_cover(manifest, ident) cover = @sectionsplit .section_split_cover(manifest, @parent.dir_name_cleanse(ident), one_doc_collection?) @files[ident][:out_path] = cover { attachment: true, index: false, out_path: cover, ref: File.join(File.dirname(manifest.file), cover) } end |
#add_section_split_instance(file, manifest, key, idx) ⇒ Object
51 52 53 54 55 56 57 58 59 60 61 62 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 51 def add_section_split_instance(file, manifest, key, idx) presfile, newkey, xml = add_section_split_instance_prep(file, key) manifest[newkey] = { parentid: key, presentationxml: true, type: "fileref", rel_path: file[:url], out_path: File.basename(file[:url]), anchors: read_anchors(xml), ids: read_ids(xml), sectionsplit_output: true, bibdata: @files[key][:bibdata], ref: presfile } @files_to_delete << file[:url] manifest[newkey][:bare] = true unless idx.zero? end |
#add_section_split_instance_prep(file, key) ⇒ Object
64 65 66 67 68 69 70 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 64 def add_section_split_instance_prep(file, key) presfile = File.join(File.dirname(@files[key][:ref]), File.basename(file[:url])) newkey = key("#{key.strip} #{file[:title]}") xml = Nokogiri::XML(File.read(presfile), &:huge) [presfile, newkey, xml] end |
#bibdata_process(entry, ident) ⇒ Object
42 43 44 45 46 47 48 49 50 51 52 53 |
# File 'lib/metanorma/files_lookup.rb', line 42 def bibdata_process(entry, ident) if entry[:attachment] entry[:bibdata] = Metanorma::Document.(ident).root else file, _filename = targetfile(entry, read: true) xml = Nokogiri::XML(file, &:huge) add_document_suffix(ident, xml) entry.merge!(anchors: read_anchors(xml), ids: read_ids(xml), bibdata: xml.at(ns("//bibdata")), document_suffix: xml.root["document_suffix"]) end end |
#bibitem_process(entry) ⇒ Object
55 56 57 58 59 60 |
# File 'lib/metanorma/files_lookup.rb', line 55 def bibitem_process(entry) entry[:bibitem] = entry[:bibdata].dup entry[:bibitem].name = "bibitem" entry[:bibitem]["hidden"] = "true" entry[:bibitem].at("./*[local-name() = 'ext']")&.remove end |
#cleanup_section_split_instance(key, manifest) ⇒ Object
25 26 27 28 29 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 25 def cleanup_section_split_instance(key, manifest) @files_to_delete << manifest["#{key}:index.html"][:ref] #@files[key].delete(:ids).delete(:anchors) @files[key][:indirect_key] = @sectionsplit.key end |
#each ⇒ Object
212 213 214 |
# File 'lib/metanorma/files_lookup.rb', line 212 def each @files.each end |
#each_with_index ⇒ Object
216 217 218 |
# File 'lib/metanorma/files_lookup.rb', line 216 def each_with_index @files.each_with_index end |
#file_entry(ref, identifier) ⇒ Object
rel_path is the source file address, determined relative to the YAML. out_path is the destination file address, with any references outside the working directory (../../…) truncated identifier is the id with only spaces, no nbsp
66 67 68 69 70 71 72 73 74 75 76 77 78 |
# File 'lib/metanorma/files_lookup.rb', line 66 def file_entry(ref, identifier) ref["fileref"] or return out = ref["attachment"] ? ref["fileref"] : File.basename(ref["fileref"]) out1 = @disambig.source2dest_filename(out) ret = if ref["fileref"] { type: "fileref", ref: @documents[Util::key identifier].file, rel_path: ref["fileref"], url: ref["url"], out_path: out1 } # @disambig.source2dest_filename(out) } else { type: "id", ref: ref["id"] } end file_entry_copy(ref, ret) ret.compact end |
#file_entry_copy(ref, ret) ⇒ Object
80 81 82 83 84 85 |
# File 'lib/metanorma/files_lookup.rb', line 80 def file_entry_copy(ref, ret) %w(attachment sectionsplit index presentation-xml url bare-after-first).each do |s| ret[s.gsub("-", "").to_sym] = ref[s] if ref[s] end end |
#get(ident, attr = nil) ⇒ Object
202 203 204 205 206 |
# File 'lib/metanorma/files_lookup.rb', line 202 def get(ident, attr = nil) if attr then @files[key(ident)][attr] else @files[key(ident)] end end |
#key(ident) ⇒ Object
194 195 196 |
# File 'lib/metanorma/files_lookup.rb', line 194 def key(ident) @c.decode(ident).gsub(/(\p{Zs})+/, " ").sub(/^metanorma-collection /, "") end |
#keys ⇒ Object
198 199 200 |
# File 'lib/metanorma/files_lookup.rb', line 198 def keys @files.keys end |
#ns(xpath) ⇒ Object
220 221 222 |
# File 'lib/metanorma/files_lookup.rb', line 220 def ns(xpath) @isodoc.ns(xpath) end |
#one_doc_collection? ⇒ Boolean
40 41 42 43 44 45 46 47 48 49 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 40 def one_doc_collection? return false docs = 0 @files.each_value do |v| v[:attachment] and next v[:presentationxml] and next docs += 1 end docs > 1 end |
#process_section_split_instance(key, manifest) ⇒ Object
15 16 17 18 19 20 21 22 23 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 15 def process_section_split_instance(key, manifest) s, sectionsplit_manifest = sectionsplit(@files[key][:ref], @files[key][:out_path], key) s.each_with_index do |f1, i| add_section_split_instance(f1, manifest, key, i) end manifest["#{key}:index.html"] = add_section_split_cover(sectionsplit_manifest, key) end |
#read_anchors(xml) ⇒ Object
map locality type and label (e.g. “clause” “1”) to id = anchor for a document Note: will only key clauses, which have unambiguous reference label in locality. Notes, examples etc with containers are just plunked against UUIDs, so that their IDs can at least be registered to be tracked as existing.
163 164 165 166 167 168 169 170 |
# File 'lib/metanorma/files_lookup.rb', line 163 def read_anchors(xml) xrefs = @isodoc.xref_init(@lang, @script, @isodoc, @isodoc.i18n, { locale: @locale }) xrefs.parse xml xrefs.get.each_with_object({}) do |(k, v), ret| read_anchors1(k, v, ret) end end |
#read_anchors1(key, val, ret) ⇒ Object
172 173 174 175 176 177 178 179 180 181 |
# File 'lib/metanorma/files_lookup.rb', line 172 def read_anchors1(key, val, ret) val[:type] ||= "clause" ret[val[:type]] ||= {} index = if val[:container] || val[:label].nil? || val[:label].empty? UUIDTools::UUID.random_create.to_s else val[:label] end ret[val[:type]][index] = key ret[val[:type]][val[:value]] = key if val[:value] end |
#read_file(docref) ⇒ Object
33 34 35 36 37 38 39 40 |
# File 'lib/metanorma/files_lookup.rb', line 33 def read_file(docref) ident = docref.at(ns("./identifier")) i = key(@isodoc.docid_prefix(ident["type"], ident.children.to_xml)) entry = file_entry(docref, ident.children.to_xml) or return bibdata_process(entry, i) bibitem_process(entry) @files[i] = entry end |
#read_files ⇒ Object
28 29 30 31 |
# File 'lib/metanorma/files_lookup.rb', line 28 def read_files @disambig = Util::DisambigFiles.new @xml.xpath(ns("//docref")).each { |d| read_file(d) } end |
#read_ids(xml) ⇒ Object
Also parse all ids in doc (including ones which won’t be xref targets)
184 185 186 187 188 189 190 191 192 |
# File 'lib/metanorma/files_lookup.rb', line 184 def read_ids(xml) ret = {} xml.traverse do |x| x.text? and next /^semantic__/.match?(x.name) and next x["id"] and ret[x["id"]] = true end ret end |
#ref_file(ref, out, read, doc) ⇒ Object
144 145 146 147 148 149 |
# File 'lib/metanorma/files_lookup.rb', line 144 def ref_file(ref, out, read, doc) file = File.read(ref, encoding: "utf-8") if read filename = out.dup filename.sub!(/\.xml$/, ".html") if doc [file, filename] end |
#sectionsplit(file, outfile, ident) ⇒ Object
72 73 74 75 76 77 78 79 80 81 82 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 72 def sectionsplit(file, outfile, ident) @sectionsplit = Sectionsplit .new(input: file, base: outfile, dir: File.dirname(file), output: outfile, compile_opts: @parent., fileslookup: self, ident: ident, isodoc: @isodoc) coll = @sectionsplit.sectionsplit.sort_by { |f| f[:order] } xml = Nokogiri::XML(File.read(file, encoding: "UTF-8"), &:huge) [coll, @sectionsplit .collection_manifest(File.basename(file), coll, xml, nil, File.dirname(file))] end |
#set(ident, attr, value) ⇒ Object
208 209 210 |
# File 'lib/metanorma/files_lookup.rb', line 208 def set(ident, attr, value) @files[key(ident)][attr] = value end |
#targetfile(data, options) ⇒ Array<String, nil>
return file contents + output filename for each file in the collection, given a docref entry so my URL should end with html or pdf or whatever formed relative to YAML file, not input path, relative to calling function
130 131 132 133 134 135 136 137 138 |
# File 'lib/metanorma/files_lookup.rb', line 130 def targetfile(data, ) = { read: false, doc: true, relative: false }.merge() path = [:relative] ? data[:rel_path] : data[:ref] if data[:type] == "fileref" ref_file path, data[:out_path], [:read], [:doc] else xml_file data[:id], [:read] end end |
#targetfile_id(ident, options) ⇒ Object
140 141 142 |
# File 'lib/metanorma/files_lookup.rb', line 140 def targetfile_id(ident, ) targetfile(get(ident), ) end |
#url(ident, options) ⇒ Object
return citation url for file so my URL should end with html or pdf or whatever
109 110 111 112 |
# File 'lib/metanorma/files_lookup.rb', line 109 def url(ident, ) data = get(ident) data[:url] || targetfile(data, )[1] end |
#url?(ident) ⇒ Boolean
are references to the file to be linked to a file in the collection, or externally? Determines whether file suffix anchors are to be used
116 117 118 119 |
# File 'lib/metanorma/files_lookup.rb', line 116 def url?(ident) data = get(ident) or return false data[:url] end |
#url_in_css_styles(doc, document_suffix) ⇒ Object
update relative URLs, url(#…), in CSS in @style attrs (including SVG)
99 100 101 102 103 104 |
# File 'lib/metanorma/files_lookup.rb', line 99 def url_in_css_styles(doc, document_suffix) doc.xpath("//*[@style]").each do |s| s["style"] = s["style"] .gsub(%r{url\(#([^)]+)\)}, "url(#\\1_#{document_suffix})") end end |
#xml_file(id, read) ⇒ Object
151 152 153 154 155 |
# File 'lib/metanorma/files_lookup.rb', line 151 def xml_file(id, read) file = @xml.at(ns("//doc-container[@id = '#{id}']")).to_xml if read filename = "#{id}.html" [file, filename] end |