Class: Metanorma::FileLookup
- Inherits:
-
Object
- Object
- Metanorma::FileLookup
- Defined in:
- lib/metanorma/files_lookup.rb,
lib/metanorma/files_lookup_sectionsplit.rb
Overview
XML collection renderer
Instance Attribute Summary collapse
-
#files_to_delete ⇒ Object
Returns the value of attribute files_to_delete.
-
#parent ⇒ Object
Returns the value of attribute parent.
Instance Method Summary collapse
- #add_document_suffix(identifier, doc) ⇒ Object
- #add_section_split ⇒ Object
- #add_section_split_cover(manifest, ident) ⇒ Object
- #add_section_split_instance(file, manifest, key, idx) ⇒ Object
- #add_section_split_instance_prep(file, key) ⇒ Object
- #add_suffix_to_attributes(doc, suffix, tag_name, attribute_name) ⇒ Object
- #bibdata_process(entry, identifier) ⇒ Object
- #bibitem_process(entry) ⇒ Object
- #each ⇒ Object
- #each_with_index ⇒ Object
-
#file_entry(ref, identifier) ⇒ Object
rel_path is the source file address, determined relative to the YAML.
- #file_entry_copy(ref, ret) ⇒ Object
- #get(ident, attr = nil) ⇒ Object
-
#initialize(path, parent) ⇒ FileLookup
constructor
hash for each document in collection of document identifier to: document reference (fileref or id), type of document reference, and bibdata entry for that file.
- #key(ident) ⇒ Object
- #keys ⇒ Object
- #ns(xpath) ⇒ Object
- #one_doc_collection? ⇒ Boolean
-
#read_anchors(xml) ⇒ Object
map locality type and label (e.g. “clause” “1”) to id = anchor for a document Note: will only key clauses, which have unambiguous reference label in locality.
- #read_anchors1(key, val, ret) ⇒ Object
- #read_file(docref) ⇒ Object
- #read_files ⇒ Object
-
#read_ids(xml) ⇒ Object
Also parse all ids in doc (including ones which won’t be xref targets).
- #ref_file(ref, out, read, doc) ⇒ Object
- #sectionsplit(file, outfile, ident) ⇒ Object
- #set(ident, attr, value) ⇒ Object
-
#targetfile(data, options) ⇒ Array<String, nil>
return file contents + output filename for each file in the collection, given a docref entry so my URL should end with html or pdf or whatever formed relative to YAML file, not input path, relative to calling function.
- #targetfile_id(ident, options) ⇒ Object
-
#url_in_css_styles(doc, document_suffix) ⇒ Object
update relative URLs, url(#…), in CSS in @style attrs (including SVG).
- #xml_file(id, read) ⇒ Object
Constructor Details
#initialize(path, parent) ⇒ FileLookup
hash for each document in collection of document identifier to: document reference (fileref or id), type of document reference, and bibdata entry for that file
15 16 17 18 19 20 21 22 23 24 25 26 |
# File 'lib/metanorma/files_lookup.rb', line 15 def initialize(path, parent) @c = HTMLEntities.new @files = {} @parent = parent @xml = parent.xml @isodoc = parent.isodoc @path = path @compile = parent.compile @documents = parent.documents @files_to_delete = [] read_files end |
Instance Attribute Details
#files_to_delete ⇒ Object
Returns the value of attribute files_to_delete.
9 10 11 |
# File 'lib/metanorma/files_lookup.rb', line 9 def files_to_delete @files_to_delete end |
#parent ⇒ Object
Returns the value of attribute parent.
9 10 11 |
# File 'lib/metanorma/files_lookup.rb', line 9 def parent @parent end |
Instance Method Details
#add_document_suffix(identifier, doc) ⇒ Object
94 95 96 97 98 99 100 |
# File 'lib/metanorma/files_lookup.rb', line 94 def add_document_suffix(identifier, doc) document_suffix = Metanorma::Utils::to_ncname(identifier) Metanorma::Utils::anchor_attributes.each do |(tag_name, attribute_name)| add_suffix_to_attributes(doc, document_suffix, tag_name, attribute_name) end url_in_css_styles(doc, document_suffix) end |
#add_section_split ⇒ Object
4 5 6 7 8 9 10 11 12 13 14 15 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 4 def add_section_split ret = @files.keys.each_with_object({}) do |k, m| if @files[k][:sectionsplit] == "true" && !@files[k]["attachment"] s, manifest = sectionsplit(@files[k][:ref], @files[k][:out_path], k) s.each_with_index { |f1, i| add_section_split_instance(f1, m, k, i) } m["#{k}:index.html"] = add_section_split_cover(manifest, k) @files_to_delete << m["#{k}:index.html"][:ref] end m[k] = @files[k] end @files = ret end |
#add_section_split_cover(manifest, ident) ⇒ Object
17 18 19 20 21 22 23 24 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 17 def add_section_split_cover(manifest, ident) cover = @sectionsplit .section_split_cover(manifest, @parent.dir_name_cleanse(ident), one_doc_collection?) @files[ident][:out_path] = cover { attachment: true, index: false, out_path: cover, ref: File.join(File.dirname(manifest.file), cover) } end |
#add_section_split_instance(file, manifest, key, idx) ⇒ Object
36 37 38 39 40 41 42 43 44 45 46 47 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 36 def add_section_split_instance(file, manifest, key, idx) presfile, newkey, xml = add_section_split_instance_prep(file, key) manifest[newkey] = { parentid: key, presentationxml: true, type: "fileref", rel_path: file[:url], out_path: File.basename(file[:url]), anchors: read_anchors(xml), ids: read_ids(xml), sectionsplit_output: true, bibdata: @files[key][:bibdata], ref: presfile } @files_to_delete << file[:url] manifest[newkey][:bare] = true unless idx.zero? end |
#add_section_split_instance_prep(file, key) ⇒ Object
49 50 51 52 53 54 55 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 49 def add_section_split_instance_prep(file, key) presfile = File.join(File.dirname(@files[key][:ref]), File.basename(file[:url])) newkey = key("#{key.strip} #{file[:title]}") xml = Nokogiri::XML(File.read(presfile), &:huge) [presfile, newkey, xml] end |
#add_suffix_to_attributes(doc, suffix, tag_name, attribute_name) ⇒ Object
87 88 89 90 91 92 |
# File 'lib/metanorma/files_lookup.rb', line 87 def add_suffix_to_attributes(doc, suffix, tag_name, attribute_name) doc.xpath(ns("//#{tag_name}[@#{attribute_name}]")).each do |elem| elem.attributes[attribute_name].value = "#{elem.attributes[attribute_name].value}_#{suffix}" end end |
#bibdata_process(entry, identifier) ⇒ Object
42 43 44 45 46 47 48 49 50 51 52 53 54 |
# File 'lib/metanorma/files_lookup.rb', line 42 def bibdata_process(entry, identifier) if entry[:attachment] entry[:bibdata] = Metanorma::Document .(identifier).root else file, _filename = targetfile(entry, read: true) xml = Nokogiri::XML(file, &:huge) add_document_suffix(identifier, xml) entry[:anchors] = read_anchors(xml) entry[:ids] = read_ids(xml) entry[:bibdata] = xml.at(ns("//bibdata")) end end |
#bibitem_process(entry) ⇒ Object
56 57 58 59 60 61 |
# File 'lib/metanorma/files_lookup.rb', line 56 def bibitem_process(entry) entry[:bibitem] = entry[:bibdata].dup entry[:bibitem].name = "bibitem" entry[:bibitem]["hidden"] = "true" entry[:bibitem].at("./*[local-name() = 'ext']")&.remove end |
#each ⇒ Object
201 202 203 |
# File 'lib/metanorma/files_lookup.rb', line 201 def each @files.each end |
#each_with_index ⇒ Object
205 206 207 |
# File 'lib/metanorma/files_lookup.rb', line 205 def each_with_index @files.each_with_index end |
#file_entry(ref, identifier) ⇒ Object
rel_path is the source file address, determined relative to the YAML. out_path is the destination file address, with any references outside the working directory (../../…) truncated identifier is the id with only spaces, no nbsp
67 68 69 70 71 72 73 74 75 76 77 78 |
# File 'lib/metanorma/files_lookup.rb', line 67 def file_entry(ref, identifier) out = ref["attachment"] ? ref["fileref"] : File.basename(ref["fileref"]) out1 = @disambig.source2dest_filename(out) ret = if ref["fileref"] { type: "fileref", ref: @documents[identifier].file, rel_path: ref["fileref"], out_path: out1 } # @disambig.source2dest_filename(out) } else { type: "id", ref: ref["id"] } end file_entry_copy(ref, ret) ret.compact end |
#file_entry_copy(ref, ret) ⇒ Object
80 81 82 83 84 85 |
# File 'lib/metanorma/files_lookup.rb', line 80 def file_entry_copy(ref, ret) %w(attachment sectionsplit index presentation-xml bare-after-first).each do |s| ret[s.gsub("-", "").to_sym] = ref[s] if ref[s] end end |
#get(ident, attr = nil) ⇒ Object
191 192 193 194 195 |
# File 'lib/metanorma/files_lookup.rb', line 191 def get(ident, attr = nil) if attr then @files[key(ident)][attr] else @files[key(ident)] end end |
#key(ident) ⇒ Object
183 184 185 |
# File 'lib/metanorma/files_lookup.rb', line 183 def key(ident) @c.decode(ident).gsub(/(\p{Zs})+/, " ").sub(/^metanorma-collection /, "") end |
#keys ⇒ Object
187 188 189 |
# File 'lib/metanorma/files_lookup.rb', line 187 def keys @files.keys end |
#ns(xpath) ⇒ Object
209 210 211 |
# File 'lib/metanorma/files_lookup.rb', line 209 def ns(xpath) @isodoc.ns(xpath) end |
#one_doc_collection? ⇒ Boolean
26 27 28 29 30 31 32 33 34 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 26 def one_doc_collection? docs = 0 @files.each_value do |v| v[:attachment] and next v[:presentationxml] and next docs += 1 end docs > 1 end |
#read_anchors(xml) ⇒ Object
map locality type and label (e.g. “clause” “1”) to id = anchor for a document Note: will only key clauses, which have unambiguous reference label in locality. Notes, examples etc with containers are just plunked against UUIDs, so that their IDs can at least be registered to be tracked as existing.
152 153 154 155 156 157 158 159 |
# File 'lib/metanorma/files_lookup.rb', line 152 def read_anchors(xml) xrefs = @isodoc.xref_init(@lang, @script, @isodoc, @isodoc.i18n, { locale: @locale }) xrefs.parse xml xrefs.get.each_with_object({}) do |(k, v), ret| read_anchors1(k, v, ret) end end |
#read_anchors1(key, val, ret) ⇒ Object
161 162 163 164 165 166 167 168 169 170 |
# File 'lib/metanorma/files_lookup.rb', line 161 def read_anchors1(key, val, ret) val[:type] ||= "clause" ret[val[:type]] ||= {} index = if val[:container] || val[:label].nil? || val[:label].empty? UUIDTools::UUID.random_create.to_s else val[:label] end ret[val[:type]][index] = key ret[val[:type]][val[:value]] = key if val[:value] end |
#read_file(docref) ⇒ Object
33 34 35 36 37 38 39 40 |
# File 'lib/metanorma/files_lookup.rb', line 33 def read_file(docref) ident = docref.at(ns("./identifier")) i = key(@isodoc.docid_prefix(ident["type"], ident.children.to_xml)) entry = file_entry(docref, ident.children.to_xml) bibdata_process(entry, i) bibitem_process(entry) @files[i] = entry end |
#read_files ⇒ Object
28 29 30 31 |
# File 'lib/metanorma/files_lookup.rb', line 28 def read_files @disambig = Util::DisambigFiles.new @xml.xpath(ns("//docref")).each { |d| read_file(d) } end |
#read_ids(xml) ⇒ Object
Also parse all ids in doc (including ones which won’t be xref targets)
173 174 175 176 177 178 179 180 181 |
# File 'lib/metanorma/files_lookup.rb', line 173 def read_ids(xml) ret = {} xml.traverse do |x| x.text? and next /^semantic__/.match?(x.name) and next x["id"] and ret[x["id"]] = true end ret end |
#ref_file(ref, out, read, doc) ⇒ Object
133 134 135 136 137 138 |
# File 'lib/metanorma/files_lookup.rb', line 133 def ref_file(ref, out, read, doc) file = File.read(ref, encoding: "utf-8") if read filename = out.dup filename.sub!(/\.xml$/, ".html") if doc [file, filename] end |
#sectionsplit(file, outfile, ident) ⇒ Object
57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/metanorma/files_lookup_sectionsplit.rb', line 57 def sectionsplit(file, outfile, ident) @sectionsplit = Sectionsplit .new(input: file, base: outfile, dir: File.dirname(file), output: outfile, compile_options: @parent., fileslookup: self, ident: ident, isodoc: @isodoc) coll = @sectionsplit.sectionsplit.sort_by { |f| f[:order] } xml = Nokogiri::XML(File.read(file, encoding: "UTF-8"), &:huge) [coll, @sectionsplit .collection_manifest(File.basename(file), coll, xml, nil, File.dirname(file))] end |
#set(ident, attr, value) ⇒ Object
197 198 199 |
# File 'lib/metanorma/files_lookup.rb', line 197 def set(ident, attr, value) @files[key(ident)][attr] = value end |
#targetfile(data, options) ⇒ Array<String, nil>
return file contents + output filename for each file in the collection, given a docref entry so my URL should end with html or pdf or whatever formed relative to YAML file, not input path, relative to calling function
119 120 121 122 123 124 125 126 127 |
# File 'lib/metanorma/files_lookup.rb', line 119 def targetfile(data, ) = { read: false, doc: true, relative: false }.merge() path = [:relative] ? data[:rel_path] : data[:ref] if data[:type] == "fileref" ref_file path, data[:out_path], [:read], [:doc] else xml_file data[:id], [:read] end end |
#targetfile_id(ident, options) ⇒ Object
129 130 131 |
# File 'lib/metanorma/files_lookup.rb', line 129 def targetfile_id(ident, ) targetfile(get(ident), ) end |
#url_in_css_styles(doc, document_suffix) ⇒ Object
update relative URLs, url(#…), in CSS in @style attrs (including SVG)
103 104 105 106 107 108 |
# File 'lib/metanorma/files_lookup.rb', line 103 def url_in_css_styles(doc, document_suffix) doc.xpath("//*[@style]").each do |s| s["style"] = s["style"] .gsub(%r{url\(#([^)]+)\)}, "url(#\\1_#{document_suffix})") end end |
#xml_file(id, read) ⇒ Object
140 141 142 143 144 |
# File 'lib/metanorma/files_lookup.rb', line 140 def xml_file(id, read) file = @xml.at(ns("//doc-container[@id = '#{id}']")).to_xml if read filename = "#{id}.html" [file, filename] end |